@@ -17,6 +17,7 @@
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
 
+#include "gi/info.h"
 #include "gjs/macros.h"
 
 // Different roles for a GIArgument; currently used only in exception and debug
@@ -51,25 +52,25 @@ GjsArgumentFlags operator|(GjsArgumentFlags const& v1, GjsArgumentFlags const& v
 
 GJS_JSAPI_RETURN_CONVENTION
 bool gjs_value_to_callback_out_arg(JSContext* context, JS::HandleValue value,
-                                   GIArgInfo* arg_info, GIArgument* arg);
+                                   const GI::ArgInfo arg_info, GIArgument* arg);
 
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_array_to_explicit_array(JSContext* cx, JS::HandleValue value,
-                                 GITypeInfo* type_info, const char* arg_name,
-                                 GjsArgumentType arg_type, GITransfer transfer,
-                                 GjsArgumentFlags flags, void** contents,
-                                 size_t* length_p);
+bool gjs_array_to_explicit_array(JSContext*, JS::HandleValue,
+                                 const GI::TypeInfo, const char* arg_name,
+                                 GjsArgumentType, GITransfer, GjsArgumentFlags,
+                                 void** contents, size_t* length_p);
 
-size_t gjs_type_get_element_size(GITypeTag element_type, GITypeInfo* type_info);
+size_t gjs_type_get_element_size(GITypeTag element_tag, const GI::TypeInfo);
+// FIXME: GI::TypeInfo&?
 
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_value_to_gi_argument(JSContext*, JS::HandleValue, GITypeInfo*,
+bool gjs_value_to_gi_argument(JSContext*, JS::HandleValue, const GI::TypeInfo,
                               const char* arg_name, GjsArgumentType, GITransfer,
                               GjsArgumentFlags, GIArgument*);
 
 GJS_JSAPI_RETURN_CONVENTION
 bool inline gjs_value_to_gi_argument(JSContext* cx, JS::HandleValue value,
-                                     GITypeInfo* type_info,
+                                     const GI::TypeInfo type_info,
                                      GjsArgumentType argument_type,
                                      GITransfer transfer, GIArgument* arg) {
     return gjs_value_to_gi_argument(cx, value, type_info,
@@ -120,21 +121,23 @@ bool gjs_value_to_gdk_atom_gi_argument(JSContext*, JS::HandleValue, GIArgument*,
                                        const char* arg_name, GjsArgumentType);
 GJS_JSAPI_RETURN_CONVENTION
 bool gjs_value_to_interface_gi_argument(JSContext*, JS::HandleValue,
-                                        GIBaseInfo* interface_info, GITransfer,
-                                        GIArgument*, const char* arg_name,
-                                        GjsArgumentType, GjsArgumentFlags);
+                                        const GI::BaseInfo interface_info,
+                                        GITransfer, GIArgument*,
+                                        const char* arg_name, GjsArgumentType,
+                                        GjsArgumentFlags);
 GJS_JSAPI_RETURN_CONVENTION
 bool gjs_value_from_basic_gi_argument(JSContext*, JS::MutableHandleValue,
                                       GITypeTag, GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_value_from_gi_argument(JSContext*, JS::MutableHandleValue, GITypeInfo*,
-                                GjsArgumentType, GITransfer, GIArgument*);
+bool gjs_value_from_gi_argument(JSContext*, JS::MutableHandleValue,
+                                const GI::TypeInfo, GjsArgumentType, GITransfer,
+                                GIArgument*);
 
 GJS_JSAPI_RETURN_CONVENTION
 inline bool gjs_value_from_gi_argument(JSContext* cx,
                                        JS::MutableHandleValue value_p,
-                                       GITypeInfo* type_info, GIArgument* arg,
-                                       bool copy_structs) {
+                                       const GI::TypeInfo type_info,
+                                       GIArgument* arg, bool copy_structs) {
     return gjs_value_from_gi_argument(
         cx, value_p, type_info, GJS_ARGUMENT_ARGUMENT,
         copy_structs ? GI_TRANSFER_EVERYTHING : GI_TRANSFER_NOTHING, arg);
@@ -167,16 +170,15 @@ bool gjs_value_from_basic_explicit_array(JSContext*, JS::MutableHandleValue,
                                          GITypeTag element_tag, GIArgument*,
                                          size_t length);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_value_from_explicit_array(JSContext* context,
-                                   JS::MutableHandleValue value_p,
-                                   GITypeInfo* type_info, GITransfer transfer,
-                                   GIArgument* arg, int length);
+bool gjs_value_from_explicit_array(JSContext*, JS::MutableHandleValue,
+                                   const GI::TypeInfo, GITransfer, GIArgument*,
+                                   size_t length);
 
 GJS_JSAPI_RETURN_CONVENTION
 inline bool gjs_value_from_explicit_array(JSContext* context,
                                           JS::MutableHandleValue value_p,
-                                          GITypeInfo* type_info,
-                                          GIArgument* arg, int length) {
+                                          const GI::TypeInfo type_info,
+                                          GIArgument* arg, size_t length) {
     return gjs_value_from_explicit_array(context, value_p, type_info,
                                          GI_TRANSFER_EVERYTHING, arg, length);
 }
@@ -193,11 +195,12 @@ bool gjs_value_from_basic_gptrarray_gi_argument(JSContext*,
                                                 GITypeTag element_tag,
                                                 GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_gi_argument_release(JSContext*, GITransfer, GITypeInfo*,
+bool gjs_gi_argument_release(JSContext*, GITransfer, const GI::TypeInfo,
                              GjsArgumentFlags, GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
 inline bool gjs_gi_argument_release(JSContext* cx, GITransfer transfer,
-                                    GITypeInfo* type_info, GIArgument* arg) {
+                                    const GI::TypeInfo type_info,
+                                    GIArgument* arg) {
     return gjs_gi_argument_release(cx, transfer, type_info,
                                    GjsArgumentFlags::NONE, arg);
 }
@@ -229,23 +232,25 @@ void gjs_gi_argument_release_basic_out_array(GITransfer, GITypeTag element_tag,
                                              size_t length, GIArgument*);
 void gjs_gi_argument_release_byte_array(GIArgument* arg);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_gi_argument_release_out_array(JSContext*, GITransfer, GITypeInfo*,
-                                       unsigned length, GIArgument*);
-GJS_JSAPI_RETURN_CONVENTION
-bool gjs_gi_argument_release_out_array(JSContext*, GITransfer, GITypeInfo*,
+bool gjs_gi_argument_release_out_array(JSContext*, GITransfer,
+                                       const GI::TypeInfo, size_t length,
                                        GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_gi_argument_release_in_array(JSContext*, GITransfer, GITypeInfo*,
-                                      unsigned length, GIArgument*);
+bool gjs_gi_argument_release_out_array(JSContext*, GITransfer,
+                                       const GI::TypeInfo, GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_gi_argument_release_in_array(JSContext*, GITransfer, GITypeInfo*,
+bool gjs_gi_argument_release_in_array(JSContext*, GITransfer,
+                                      const GI::TypeInfo, size_t length,
                                       GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
-bool gjs_gi_argument_release_in_arg(JSContext*, GITransfer, GITypeInfo*,
+bool gjs_gi_argument_release_in_array(JSContext*, GITransfer,
+                                      const GI::TypeInfo, GIArgument*);
+GJS_JSAPI_RETURN_CONVENTION
+bool gjs_gi_argument_release_in_arg(JSContext*, GITransfer, const GI::TypeInfo,
                                     GjsArgumentFlags, GIArgument*);
 GJS_JSAPI_RETURN_CONVENTION
 inline bool gjs_gi_argument_release_in_arg(JSContext* cx, GITransfer transfer,
-                                           GITypeInfo* type_info,
+                                           const GI::TypeInfo type_info,
                                            GIArgument* arg) {
     return gjs_gi_argument_release_in_arg(cx, transfer, type_info,
                                           GjsArgumentFlags::ARG_IN, arg);
@@ -254,8 +259,6 @@ inline bool gjs_gi_argument_release_in_arg(JSContext* cx, GITransfer transfer,
 GJS_JSAPI_RETURN_CONVENTION
 bool _gjs_flags_value_is_valid(JSContext* cx, GType gtype, int64_t value);
 
-[[nodiscard]] int64_t _gjs_enum_from_int(GIEnumInfo* enum_info, int int_value);
-
 GJS_JSAPI_RETURN_CONVENTION
 bool gjs_array_from_strv(JSContext             *context,
                          JS::MutableHandleValue value_p,
@@ -269,13 +272,13 @@ bool gjs_array_to_strv (JSContext   *context,
 
 GJS_JSAPI_RETURN_CONVENTION
 bool gjs_array_from_g_value_array(JSContext* cx, JS::MutableHandleValue value_p,
-                                  GITypeInfo* param_info, GITransfer,
+                                  const GI::TypeInfo param_info, GITransfer,
                                   const GValue* gvalue);
 
 GJS_JSAPI_RETURN_CONVENTION
 bool gjs_object_from_g_hash(JSContext* cx, JS::MutableHandleValue,
-                            GITypeInfo* key_param_info,
-                            GITypeInfo* val_param_info, GITransfer transfer,
-                            GHashTable* hash);
+                            const GI::TypeInfo key_param_info,
+                            const GI::TypeInfo val_param_info,
+                            GITransfer transfer, GHashTable* hash);
 
 #endif  // GI_ARG_H_
