@@ -16,8 +16,10 @@
 #include <js/PropertySpec.h>
 #include <js/RootingAPI.h>
 #include <js/TypeDecls.h>
+#include <mozilla/Maybe.h>
 
 #include "gi/cwrapper.h"
+#include "gi/info.h"
 #include "gi/wrapperutils.h"
 #include "gjs/jsapi-util.h"
 #include "gjs/macros.h"
@@ -71,9 +73,13 @@ class InterfaceBase : public GIWrapperBase<InterfaceBase, InterfacePrototype,
 
 class InterfacePrototype
     : public GIWrapperPrototype<InterfaceBase, InterfacePrototype,
-                                InterfaceInstance, GIInterfaceInfo> {
+                                InterfaceInstance,
+                                mozilla::Maybe<GI::AutoInterfaceInfo>,
+                                mozilla::Maybe<GI::InterfaceInfo>> {
     friend class GIWrapperPrototype<InterfaceBase, InterfacePrototype,
-                                    InterfaceInstance, GIInterfaceInfo>;
+                                    InterfaceInstance,
+                                    mozilla::Maybe<GI::AutoInterfaceInfo>,
+                                    mozilla::Maybe<GI::InterfaceInfo>>;
     friend class GIWrapperBase<InterfaceBase, InterfacePrototype,
                                InterfaceInstance>;
     friend class InterfaceBase;  // for has_instance_impl
@@ -81,9 +87,7 @@ class InterfacePrototype
     // the GTypeInterface vtable wrapped by this JS object
     GTypeInterface* m_vtable;
 
-    static constexpr InfoType::Tag info_type_tag = InfoType::Interface;
-
-    explicit InterfacePrototype(GIInterfaceInfo* info, GType gtype);
+    explicit InterfacePrototype(mozilla::Maybe<const GI::InterfaceInfo>, GType);
     ~InterfacePrototype(void);
 
     // JSClass operations
