@@ -11,6 +11,7 @@
 #include <stdint.h>  // for uint32_t
 
 #include <functional>
+#include <unordered_set>
 #include <vector>
 
 #include <girepository.h>
@@ -33,6 +34,7 @@
 #include "gi/wrapperutils.h"
 #include "gjs/auto.h"
 #include "gjs/jsapi-util-root.h"
+#include "gjs/jsapi-util.h"  // for gjs_throw
 #include "gjs/macros.h"
 #include "util/log.h"
 
@@ -90,8 +92,7 @@ class ObjectBase
                                         GIArgument* arg,
                                         GIDirection transfer_direction,
                                         GITransfer transfer_ownership,
-                                        GType expected_gtype,
-                                        GIBaseInfo* expected_info = nullptr);
+                                        GType expected_gtype);
 
  private:
     // This is used in debug methods only.
@@ -108,15 +109,26 @@ class ObjectBase
     [[nodiscard]] bool is_custom_js_class();
 
  public:
-    GJS_JSAPI_RETURN_CONVENTION
-    static bool typecheck(JSContext* cx, JS::HandleObject obj,
-                          GIObjectInfo* expected_info, GType expected_gtype);
-    [[nodiscard]] static bool typecheck(JSContext* cx, JS::HandleObject obj,
-                                        GIObjectInfo* expected_info,
-                                        GType expected_gtype,
-                                        GjsTypecheckNoThrow no_throw) {
-        return GIWrapperBase::typecheck(cx, obj, expected_info, expected_gtype,
-                                        no_throw);
+    // Overrides GIWrapperBase::typecheck(). We only override the overload that
+    // throws, so that we can throw our own more informative error.
+    template <typename T>
+    GJS_JSAPI_RETURN_CONVENTION static bool typecheck(JSContext* cx,
+                                                      JS::HandleObject obj,
+                                                      T expected) {
+        if (GIWrapperBase::typecheck(cx, obj, expected))
+            return true;
+
+        gjs_throw(cx,
+                  "This JS object wrapper isn't wrapping a GObject."
+                  " If this is a custom subclass, are you sure you chained"
+                  " up to the parent _init properly?");
+        return false;
+    }
+    template <typename T>
+    [[nodiscard]]
+    static bool typecheck(JSContext* cx, JS::HandleObject obj, T expected,
+                          GjsTypecheckNoThrow no_throw) {
+        return GIWrapperBase::typecheck(cx, obj, expected, no_throw);
     }
 
     /* JSClass operations */
@@ -201,9 +213,12 @@ struct IdHasher {
 };
 
 class ObjectPrototype
-    : public GIWrapperPrototype<ObjectBase, ObjectPrototype, ObjectInstance> {
-    friend class GIWrapperPrototype<ObjectBase, ObjectPrototype,
-                                    ObjectInstance>;
+    : public GIWrapperPrototype<ObjectBase, ObjectPrototype, ObjectInstance,
+                                mozilla::Maybe<GI::AutoObjectInfo>,
+                                mozilla::Maybe<GI::ObjectInfo>> {
+    friend class GIWrapperPrototype<ObjectBase, ObjectPrototype, ObjectInstance,
+                                    mozilla::Maybe<GI::AutoObjectInfo>,
+                                    mozilla::Maybe<GI::ObjectInfo>>;
     friend class GIWrapperBase<ObjectBase, ObjectPrototype, ObjectInstance>;
 
     using NegativeLookupCache =
@@ -211,16 +226,14 @@ class ObjectPrototype
 
     NegativeLookupCache m_unresolvable_cache;
     // a list of vfunc GClosures installed on this prototype, used when tracing
-    std::vector<GClosure*> m_vfuncs;
+    std::unordered_set<GClosure*> m_vfuncs;
     // a list of interface types explicitly associated with this prototype,
     // by gjs_add_interface
     std::vector<GType> m_interface_gtypes;
 
-    ObjectPrototype(GIObjectInfo* info, GType gtype);
+    ObjectPrototype(mozilla::Maybe<GI::ObjectInfo> info, GType gtype);
     ~ObjectPrototype();
 
-    static constexpr InfoType::Tag info_type_tag = InfoType::Object;
-
  public:
     [[nodiscard]] static ObjectPrototype* for_gtype(GType gtype);
 
@@ -231,8 +244,8 @@ class ObjectPrototype
     GJS_JSAPI_RETURN_CONVENTION
     bool get_parent_constructor(JSContext* cx,
                                 JS::MutableHandleObject constructor) const;
-
-    [[nodiscard]] bool is_vfunc_unchanged(GIVFuncInfo* info);
+    [[nodiscard]]
+    bool is_vfunc_unchanged(const GI::VFuncInfo) const;
     static void vfunc_invalidated_notify(void* data, GClosure* closure);
 
     GJS_JSAPI_RETURN_CONVENTION
@@ -266,7 +279,7 @@ class ObjectPrototype
 
     GJS_JSAPI_RETURN_CONVENTION
     static bool define_class(JSContext* cx, JS::HandleObject in_object,
-                             GIObjectInfo* info, GType gtype,
+                             mozilla::Maybe<const GI::ObjectInfo>, GType,
                              GType* interface_gtypes,
                              uint32_t n_interface_gtypes,
                              JS::MutableHandleObject constructor,
@@ -419,7 +432,7 @@ class ObjectInstance : public GIWrapperInstance<ObjectBase, ObjectPrototype,
     /* Methods to manipulate the linked list of instances */
 
  private:
-    static std::vector<ObjectInstance*> s_wrapped_gobject_list;
+    static std::unordered_set<ObjectInstance*> s_wrapped_gobject_list;
     void link(void);
     void unlink(void);
     [[nodiscard]] static size_t num_wrapped_gobjects() {
@@ -493,9 +506,13 @@ class ObjectInstance : public GIWrapperInstance<ObjectBase, ObjectPrototype,
                    JS::HandleObject obj);
     [[nodiscard]] const char* to_string_kind() const;
 
-    GJS_JSAPI_RETURN_CONVENTION
-    bool typecheck_impl(JSContext* cx, GIBaseInfo* expected_info,
-                        GType expected_type) const;
+    // Overrides GIWrapperInstance::typecheck_impl()
+    template <typename T>
+    GJS_JSAPI_RETURN_CONVENTION bool typecheck_impl(T expected) const {
+        g_assert(m_gobj_disposed || !m_ptr ||
+                 gtype() == G_OBJECT_TYPE(m_ptr.as<GObject*>()));
+        return GIWrapperInstance::typecheck_impl(expected);
+    }
 
     /* Notification callbacks */
     void gobj_dispose_notify(void);
